<?php

namespace App\Service;

use App\Exception;

use SimpleXMLElement;

class IntegrationService {
    
    private $util;
    private $integrationFactory;
    private $consumerFactory;

    public function __construct (\App\Util\Util $util,
                                 \App\Factory\IntegrationFactory $integrationFactory,
                                 \App\Factory\ConsumerFactory $consumerFactory) {

        $this->util                 = $util;
        $this->integrationFactory   = $integrationFactory;
        $this->ConsumerFactory      = $consumerFactory;
    }
    
    public function saveLog($fileName, $content, $logType) {
        try {
            $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
            $config       = json_decode($configsFile, true);
            
            $logPath = $config['logPath'];
            
            $folder = $logPath . "/$logType/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = $folder . "/" . $fileName;
            
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, $content, FILE_APPEND);
        } catch(\Exception $e) { 
            
        }
    }
    
    public function getConsumer($clientCode, $consumerCode) {
        return $this->ConsumerFactory->getConsumer($clientCode, $consumerCode);
    }
    
    public function getBalance($clientCode, $consumerCode, $campaignCode) {
        $balance = $this->integrationFactory->getBalance($clientCode, $consumerCode, $campaignCode);
        
        $return = [
            'consumerName' => isset($balance['NMCONSUMIDOR']) ? $balance['NMCONSUMIDOR'] : '',
            'consumerCode' => $consumerCode,
            'clientCode'   => $clientCode,
            'campaignCode' => $campaignCode,
            'balance'      => isset($balance['VRSALDOCONS']) ? $this->util->toFloat($balance['VRSALDOCONS']) : 0
        ];
         
        return $return;
    }
    
    public function registerConsumer($subsidiaryCode, $clientCode, $consumerCode, $phone) {
        $data = [
            'massage'  => '',
            'data'     => []
        ];
        
        $isExist = $this->ConsumerFactory->getConsumer($clientCode, $consumerCode);
        
        if(!empty($isExist)) {
            $data['message'] = 'Consumidor já cadastrado na base.';
            $data['data'] = $isExist;
            return $data;
        }
        
        $generalParams = $this->ConsumerFactory->getGeneralParams($subsidiaryCode);
        
        $generalParams['CDFILIAL']     = $subsidiaryCode;
        $generalParams['CDCLIENTE']    = $clientCode;
        $generalParams['CDCONSUMIDOR'] = $consumerCode;
        $generalParams['NRTELECONS']   = $phone;
        
        $this->insertConsumer($generalParams);
        
        $data['message'] = "Consumidor {$clientCode} - {$consumerCode} inserido no BD.";
        $data['data']    = $this->ConsumerFactory->getConsumer($clientCode, $consumerCode);
        
        return $data;
    }
    
    public function insertConsumer($params) {
        $consumerDetails = [
            'CDCLIENTE'    => $params['CDCLIENTE'],
            'CDCONSUMIDOR' => $params['CDCONSUMIDOR'],
            'NMCONSUMIDOR' => isset($params['NMCONSUMIDOR']) ? $params['NMCONSUMIDOR'] : 'NOVO CONSUMIDOR',
            'CDCCUSCLIE'   => $params['CDCCUSCLIE'],
            'CDTIPOCONS'   => $params['CDTIPOCONS'],
            'CDIDCONSUMID' => $params['NRTELECONS'],
            'NRTELECONS'   => isset($params['NRTELECONS']) ? $params['NRTELECONS'] : null,
            'NRCPFRESPCON' => isset($params['NRCPFRESPCON']) ? $params['NRCPFRESPCON'] : null
        ];
        
        $this->ConsumerFactory->insertConsumer($consumerDetails);
    }
    
    public function processScore($subsidiaryCode, $posCode, $checkRegConsumer, $products = null, $saleSeq, $dtAbertur) {
        $today = date('d/m/Y');
        $campaigns = $this->integrationFactory->getCampaigns($subsidiaryCode, $today);
        
        
        foreach($campaigns as $campaign) {
            $totalScore = 0;
            if(isset($campaign['IDCAMPANHA']) && $campaign['IDCAMPANHA'] == 'S') {
                $expiration = $campaign['NRDIASVALPONTO'];
                
                foreach($products as $product) {
                    $percent = $this->integrationFactory->getPercentValuePerProduct($subsidiaryCode, $campaign['CDCAMPANHA'], $product['productCode'], $today);
                    
                    if(empty($percent))
                        continue;
                        
                    $score = bcdiv(bcmul($this->util->toFloat($product['totalValue']), $this->util->toFloat($percent), 2), '100', 2);
                    $totalScore = bcadd($totalScore, $score, 2);
                }
                
                if($totalScore > 0) {
                    if(!empty($expiration)) {
                        $expiration = date("d-m-Y", strtotime("+{$expiration} days"));
                        $endCampaign = strtotime(str_replace("/", "-", $campaign['DTFIMVIGCAMP']));

                        if(strtotime($expiration) > $endCampaign) {
                            $expiration = $campaign['DTFIMVIGCAMP'];
                        }
                    }
                    $obs = 'Pontuação da venda. (fidelity-api)';
                    $nrSeqMovCons = $this->util->newCode("MOVPONTOCONS".$campaign['CDCAMPANHA'].$checkRegConsumer['CDCLIENTE'].$checkRegConsumer['CDCONSUMIDOR'], 10);
                    $this->integrationFactory->insertConsumerScore($subsidiaryCode, $posCode, $checkRegConsumer['CDCLIENTE'], $checkRegConsumer['CDCONSUMIDOR'], $campaign, $nrSeqMovCons, $totalScore, $expiration, $saleSeq, $dtAbertur, $obs);
                    $balance = $this->getBalance($checkRegConsumer['CDCLIENTE'], $checkRegConsumer['CDCONSUMIDOR'], $campaign['CDCAMPANHA']);
                    
                    $insertScore = [
                        'campaignCode'     => $campaign['CDCAMPANHA'],
                        'campaignId'       => $campaign['IDCAMPANHA'],
                        'campaignName'     => $campaign['DSCAMPANHA'],
                        'generatedBalance' => $totalScore,
                        'totalBalance'     => $balance['balance'],
                        'expiration'       => str_replace("-", "/", $expiration)
                    ];
                }
            } 
        }
        
        return $insertScore;
    }
    
    public function redeemBalance($subsidiaryCode, $posCode, $consumerDetails, $campaignCode, $balanceUsed, $saleSeq, $dtAbertur) {
        $today = date('d/m/Y');
        
        $movements = $this->integrationFactory->getCreditsInput($consumerDetails, $campaignCode, $today);

        if(!count($movements))
            throw new Exception\ParamsException("Consumidor não possui credito para retirada de saldo.");
        
        $totalDiscount = 0;
        $remaining = $balanceUsed;

        foreach($movements as $movement) {
            if($movement['IDCAMPANHA'] == 'S') {
                if($totalDiscount == $balanceUsed) {
                    continue;
                }
                
                $vrSaldoCons = $this->util->toFloat($movement['VRSALDOCONS']);
                $discount  = $vrSaldoCons >= $remaining ?  $remaining : $vrSaldoCons;
                $value     = $this->util->toFloat(bcsub($vrSaldoCons, $discount, 2));
                
                $this->integrationFactory->updateMovPontoCons($movement, $value);
                $totalDiscount = $this->util->toFloat(bcadd($totalDiscount, $discount, 2));
                $remaining = $this->util->toFloat(bcsub($balanceUsed, $totalDiscount, 2));
            }
        }
        
        if($totalDiscount) {
            $nrSeqMovCons = $this->util->newCode("MOVPONTOCONS".$campaignCode.$consumerDetails['CDCLIENTE'].$consumerDetails['CDCONSUMIDOR'], 10);
            $this->integrationFactory->insertRedeemBalance($subsidiaryCode, $posCode, $consumerDetails['CDCLIENTE'], $consumerDetails['CDCONSUMIDOR'], $campaignCode, $nrSeqMovCons, $balanceUsed, $saleSeq, $dtAbertur);
        }
    }
    
    public function getBranch($subsidiaryCode) {
        return $this->integrationFactory->getBranchDetails($subsidiaryCode);
    }
    
    public function cancelBalance($subsidiaryCode, $posCode, $consumerDetails, $saleSeq, $dtabertur) {
        $movements = $this->integrationFactory->getMovementConsumer($subsidiaryCode, $posCode, $consumerDetails, $saleSeq, $dtabertur);
        
        if(empty($movements))
            throw new Exception\ParamsException("Venda não encontrada. Verifique os parametros enviados");
            
        foreach($movements as $movement) {
            if($movement['IDCAMPANHA'] == 'S') {
                if($movement['IDTPMOVCONS'] == 'C') {
                    $value = 0;
                    $movement['IDTPMOVCONS'] = 'X';
                    $this->integrationFactory->updateMovPontoCons($movement, $value);
                }
                
                if($movement['IDTPMOVCONS'] == 'D') {
                    $movement['IDTPMOVCONS'] = 'X';
                    $value = $this->util->toFloat($movement['VRMOVPONTOCONS']);
                    $this->integrationFactory->updateMovPontoCons($movement, 0);
                    
                    $expiration = $movement['NRDIASVALPONTO'];
                    if(!empty($expiration)) {
                        $expiration = date("d-m-Y", strtotime("+{$expiration} days"));
                        $endCampaign = strtotime(str_replace("/", "-", $movement['DTFIMVIGCAMP']));
                        
                        if(strtotime($expiration) > $endCampaign) {
                            $expiration = $movement['DTFIMVIGCAMP'];
                        }
                    }
                    
                    $obs = 'Restituição de saldo. (fidelity-api)';
                    $nrSeqMovCons = $this->util->newCode("MOVPONTOCONS".$movement['CDCAMPANHA'].$movement['CDCLIENTE'].$movement['CDCONSUMIDOR'], 10);
                    $this->integrationFactory->insertConsumerScore($subsidiaryCode, $posCode, $movement['CDCLIENTE'], $movement['CDCONSUMIDOR'], $movement, $nrSeqMovCons, $value, $expiration, $saleSeq, $movement['DTABERTUR'], $obs);
                }
            }
        }
    }
    
    public function getConsumerById($id, $type) {
        $queryField = $this->consumerKeyTypes($type);
        if(empty($queryField))
            throw new Exception\ParamsException("Tipo de chave para consulta é invalido. Revise a documentação para obter os tipos de chave aceitos.");
        
        if($type == 'phone')
            $id = preg_replace('/[\(\)\s-]/', '', $id);
        
        if($type == 'cpf')
            $id = preg_replace('/[\.-]/', '', $id);
        
        $consumer = $this->ConsumerFactory->getConsumerById($id, $queryField);
        
        if(empty($consumer))
            throw new Exception\ParamsException("Consumidor não encontrado.");
            
            
        return $consumer;
    }
    
    private function consumerKeyTypes($type) {
        $types = [
            'cpf'    => 'NRCPFRESPCON',
            'email'  => 'DSEMAILCONS',
            'phone'  => 'NRTELECONS',
            'cracha' => 'CDIDCONSUMID'
        ];
        
        return isset($types[$type]) ? $types[$type] : null;
    }
    
    public function updateConsumer($data) {
        $fields = [
            'clientCode',
            'consumerCode',
            'name',
            'email',
            'phone',
            'cpf',
            'birthday',
            'gender',
            'cep',
            'street',
            'neighborhood',
            'addressNumber',
            'complement',
            'state',
            'city'
        ];
        
        foreach($fields as $field) {
            if(!isset($data[$field]))
                throw new Exception\ParamsException("Campo {$field} não enviado.");
        }
        
        if(!empty($data['state']) && !empty($data['city']))
            $data['CDMUNICIPIO'] = $this->ConsumerFactory->getMunicipio($data['state'], $data['city'])['CDMUNICIPIO'];
        
        $this->ConsumerFactory->updateConsumer($data);
    }
    
    public function getMovementConsumer($subsidiaryCode, $posCode, $consumerDetails, $saleSeq, $dtabertur) {
        return $this->integrationFactory->getMovementConsumer($subsidiaryCode, $posCode, $consumerDetails, $saleSeq, $dtabertur);
    }
}
